/* filename:    confprio.g
** description: configures the prior for the nuisance parameters
*/


/********************************************************
**      Configuration of Prior for
**      baseline individual.
**      We put a prior on the log-wage in 1976      
*/

/* Baseline wage
*/
lwage76_m = ln(10);
lwage76_s = 0.2;
wage76_m  = exp( lwage76_m  + 0.5*lwage76_s^2*scale_g^2 ) ;

/* Age-Earnings Profile
*/
AEPscale_m = 0.4;
AEPscale_s = 0.2;

AEPpeak_m = 55;
AEPpeak_s =  5;

/* Gender bias
*/
gender_m = -0.4;
gender_s =  0.2;

/* Return on Education
*/ 
retedu_m = 0.1;
retedu_s = 0.1;

wagegr_m   = 0.02;
wagegr_s   = 0.004;

wagein_s   = 0.01;

lspecific_m = 0;
lspecific_s = 0.25;
specific_m  = exp( lspecific_m + 0.5*lspecific_s^2*scale_b^2);


/********************************************************
**      Generate a prior for the panel model coefficients
**
*/

if mspec == 1;

  gamss1_m = lwage76_m;
  gamss1_v = lwage76_s^2;

  DUMMY77select = 1;

else;

  /* use delta-method to convert AEP coefficients
  ** coefficients on a(t)^1, a(t)^2
  */
  gamAEP_m = ( 2*AEPscale_m*AEPpeak_m | -AEPscale_m ) / (AGE_b - AEPpeak_m)^2;

  trvar    = ( ( 2*AEPpeak_m ~ ( 2*AEPscale_m + 4*AEPscale_m*AEPpeak_m/(AGE_b - AEPpeak_m) ) )|
               ( -1 ~ -2*AEPscale_m/(AGE_b - AEPpeak_m) )) / (AGE_b - AEPpeak_m)^2;
  gamAEP_v = trvar* ( (AEPscale_s^2 ~ 0)|(0 ~ AEPpeak_s^2) )* trvar';
  
  /* Merge prior for AEP coefficient with prior for the other exogenous vars:
  ** AGE, AGE^2, EDUC, GENDER, WAGE76
  */
  gamss1_m = gamAEP_m | retedu_m | gender_m | lwage76_m;
  gamss1_v = ( gamAEP_v   ~ zeros(2,3)         )|
             ( zeros(1,2) ~ retedu_s^2 ~ 0 ~ 0 )|
             ( zeros(1,2) ~ 0 ~ gender_s^2 ~ 0 )|
             ( zeros(1,2) ~ 0 ~ 0 ~ lwage76_s^2 );            

  DUMMY77select = (0 ~ 0 ~ 0 ~ 0 ~ 1 ); 

endif;

/* Specify the prior for DUMMY77, 79, 80, ... 92 conditional
** on gamss1
*/
yearseq   = seqa(1,1,nYDUMMIES);
epsweight = ones(nYDUMMIES, nYDUMMIES);

i = 2;
do until i > nYDUMMIES;
   epsweight[i,i:nYDUMMIES] = i*ones(1,nYDUMMIES-i+1);
   epsweight[i:nYDUMMIES,i] = i*ones(nYDUMMIES-i+1,1);
   i = i+1;
endo;

gamss2c1_ma = wagegr_m * yearseq;
gamss2c1_mb = ones(nYDUMMIES,1).*.DUMMY77select;
gamss2c1_v  = wagegr_s^2 * ( yearseq*yearseq') + wagein_s*epsweight;

/* Generate the joint prior
*/
gamss21_v = gamss2c1_mb*gamss1_v;
gamss2_m  = gamss2c1_ma + gamss2c1_mb*gamss1_m;
gamss2_v  = gamss2c1_v  + gamss2c1_mb*gamss1_v*gamss2c1_mb';
             
gamss_m0  = gamss1_m | gamss2_m;
gamss_v0  = ( gamss1_v ~ gamss21_v')|(gamss21_v ~ gamss2_v);

beta_m0  = lspecific_m;
beta_v0  = lspecific_s^2;

