/* filename:    convdata.g
** description: loads the PSID data into Gauss and saves various
**              subsets
*/

new;
closeall;
library user, lbdlib, pgraph;
cls;

lpath = "c:\\projects\\active\\persist\\data_p\\";
ldata  = lpath $+ "lbdpanel";

open fhdata = ^ldata for read;

eofloop = 0;
nread   = 1000;
nobs    = 1;
paneldata       = readr(fhdata, 1);
paneldata_names = getname(ldata);

do until eofloop; 

   datarows = readr( fhdata, nread );
   paneldata = paneldata | datarows;
   nobs = nobs + rows(datarows);

   locate 1,1;
   "Part 1";
   "Number of Observations Read " nobs;

   eofloop = eof(fhdata);
  
endo;

"The following series were read";
$paneldata_names';

"";
"Press key to continue";
k = keyw;


/* Compute number of observations for each individual
*/
cls;

currentid = paneldata[1,1];
individual_pointer = 1;
individual_counter = 1;
individual_nobs = 0;
individual_nobs_vec = zeros(nobs,1);
individual_age = paneldata[1,4];

i = 2;
do until i > nobs;

    if paneldata[i,1] NE currentid;
       individual_nobs_vec[individual_pointer[rows(individual_pointer)]:i-1] 
        = ones(i - individual_pointer[rows(individual_pointer)],1)*(i - individual_pointer[rows(individual_pointer)]);
       individual_nobs = individual_nobs | (i - individual_pointer[rows(individual_pointer)]);
       paneldata[individual_pointer[rows(individual_pointer)]:i-1,4] = individual_age;
       currentid = paneldata[i,1];
       individual_age = paneldata[i,4];
       individual_pointer = individual_pointer | i;
       individual_counter = individual_counter + 1;
    else;
       individual_age = individual_age | individual_age[rows(individual_age)]+ paneldata[i,2] - paneldata[i-1,2];
    endif;

    locate 1,1;
    "Step" i "of" nobs;
    i = i+1;
endo;

closeall fhdata;

individual_nobs_vec[individual_pointer[rows(individual_pointer)]:i-1] 
= ones(i - individual_pointer[rows(individual_pointer)],1)*(i - individual_pointer[rows(individual_pointer)]);

individual_nobs = individual_nobs | (i - individual_pointer[rows(individual_pointer)]);
individual_nobs = individual_nobs[2:rows(individual_nobs)];

"Number of individuals               " Individual_counter;
"Average number of obs per individual" nobs/individual_counter;  
"Press key to continue";
k = keyw;

/* Save the panel data as follows
** columns 1,2 : ID
** columns 3-5 : charac
** columns 6,7 : wages
** columns 8-14: hours
** columns 15- : ydummies
*/

odata = lpath $+ "lbdid";
vnames = paneldata_names[1:2];
create fhdata = ^odata with ^vnames, 2, 4;
writer(fhdata, paneldata[.,1:2]);
closeall fhdata;

odata = lpath $+ "lbdchar";
vnames = paneldata_names[4] | paneldata_names[4] $+ "2" | paneldata_names[3] | paneldata_names[5];
create fhdata = ^odata with ^vnames, 4, 8;
writer(fhdata, paneldata[.,4]~paneldata[.,4]^2~paneldata[.,3]~paneldata[.,5]);
closeall fhdata;

odata = lpath $+ "lbdwages";
vnames = paneldata_names[6:7];
create fhdata = ^odata with ^vnames, 2, 8;
writer(fhdata, paneldata[.,6:7]);
closeall fhdata;
  
odata = lpath $+ "lbdhours";
vnames = paneldata_names[8:14];
create fhdata = ^odata with ^vnames, 7, 8;
writer(fhdata, paneldata[.,8:14]);
closeall fhdata;

odata = lpath $+ "lbdydum";
vnames = paneldata_names[15:cols(paneldata)];
create fhdata = ^odata with ^vnames, cols(paneldata)-14, 8;
writer(fhdata, paneldata[.,15:cols(paneldata)]);
closeall fhdata;

odata = lpath $+ "lbdti";
vnames = { T(i) };
create fhdata = ^odata with ^vnames, 1, 4;
writer(fhdata, individual_nobs);
closeall fhdata;

odata = lpath $+ "lbdvecti";
vnames = { vec(T(i)) };
create fhdata = ^odata with ^vnames, 1, 4;
writer(fhdata, individual_nobs_vec);
closeall fhdata;

