/* filename:    lbdhes_p.g
** description: The program computes the hessian at the posterior mode 
** created:     05/30/01
*/

new;
closeall;
cls;
outwidth 128;
library user, pgraph, lbdlib;

/********************************************************
**      List of Global Variables
**
**      ww, xx, xw, wiiw, xiix, xiiw, iw, ix, w_names, x_names, ti
**      nx, nw, nchar, nydummies
**      gamss_m0, gamss_v0, beta_m0, beta_v0
**      age_base, educ_base, gender_base, hours_base, xbaseline
*/
#include c:\projects\active\persist\gauss\prog_p03\confspec.g;

/********************************************************
**      Load "sufficient statistics" for panel data set
**
*/
lpath = "c:\\projects\\active\\persist\\data_p\\spec" $+ ftos(mspec,"%*.*lf",1,0);
loadm path = ^lpath wiiw, xiiw, xiix, iw, ix,
                    ww, xx, xw, w_names, x_names, ti,
                    nchar, nydummies;
nx = cols(xx);
nw = cols(ww);
"Data Matrices successfully loaded";
"";
"Number of Observations: "  sumc(ti);
"Number of Individuals:  "  rows(ti);
"Regressors X:           "; $x_names';
"Right-hand-side Vars:   "; $w_names';
"";
"";

/********************************************************
**      Configure the prior for the nuisance parameters
**
**      specify baseline ind: age_base, educ_base, gender_base, hours_base     
**                            xbaseline
**      individual effect:    beta_m0, beta_v0
**      covariates       :    gamss_m0, gamss_v0
*/
#include c:\projects\active\persist\gauss\prog_p03\confprio.g;
gamss_v0 = gamss_v0*scale_g^2;
beta_v0  = beta_v0*scale_b^2;
"Prior successfully configured";
"";

"Baseline Individual";
"===================";
"AGE:                         " AGE_B;
"EDUCATION:                   " EDUC_B;
"GENDER (0 is MALE):          " GENDER_B;
"HOURS:                       " HOURS_B;
"EXP(Prior Mean lWAGE (1976)):" exp(lwage76_m);
"Prior Mean WAGE (1976):      " wage76_m;
"Prior Mean Ind Effect:       " specific_m;
"";
"Scaling factors for covariance matrices:  " scale_b~scale_g;
"S.D. for individual specific effect is:   " sqrt(beta_v0);
"";
"Press Key to Continue";
k = keyw;

/********************************************************
**   Load Model parameters, Posterior Mode
*/
cls;
lpath = "c:\\projects\\active\\persist\\gauss\\prog_p03\\para";
loadm path = ^lpath lbdmod_p, para_names;
npara = rows(lbdmod_p);

"Parameter   | Estimate ";
outmat = para_names~lbdmod_p;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);


/* Load Prior for Structural Parameters
*/
priorfile = "panelpri.out";
load path=^lpath prior[npara,3] = ^priorfile;

pmean  = prior[.,1];
pstdd  = prior[.,2];
pshape = prior[.,3];


para = lbdmod_p;
"Prior*Likelihood at Mode";
fcn(para);
"Press Key to Continue";
k = keyw;
cls;

/*
end;
goto evalhess;
*/

/* Compute Hessian, element by element, fine tune with dxscale
*/
comphess:

ndx = 6;
dx =  exp(-seqa(6,2,ndx));
hessian = zeros( npara, npara );
gradx = zeros(ndx,1);
grady = zeros(ndx,1);
gradxy = zeros(ndx,1);
hessdiag = zeros(ndx,1);
dxscale = ones(npara,1);


/* Compute Diagonal elements first
*/
seli = 1;
do until seli > npara;
     locate 1,1;
     "Hessian Element    (" seli seli ")";
     i=1;
     do until i > ndx;
      paradx = para;
      parady = para;
      paradx[seli] = paradx[seli] + dx[i]*dxscale[seli];
      parady[seli] = parady[seli] - dx[i]*dxscale[seli];
      paradxdy = paradx;
      paradxdy[seli] = paradxdy[seli] - dx[i]*dxscale[seli];
      fx  = fcn(para);
      fdx = fcn(paradx);
      fdy = fcn(parady);
      fdxdy = fcn(paradxdy);
      gradx[i] = -( fx - fdx )/ (dx[i]*dxscale[seli]);
      grady[i] = ( fx - fdy )/ (dx[i]*dxscale[seli]);
      gradxy[i] = -(fx -fdxdy)/ sqrt( (dx[i]*dxscale[seli])^2 + (dx[i]*dxscale[seli])^2 );
      hessdiag[i] = -( 2*fx - fdx - fdy)/(dx[i]*dxscale[seli])^2; 
      hessdiag[i] = -( fx - fdx - fdy + fdxdy )/(dx[i]*dx[i]*dxscale[seli]*dxscale[seli]);
      i = i+1;
     endo;
     "Values";
     -hessdiag';
     hessian[seli,seli] = -0.5*(hessdiag[3]+hessdiag[4]);
     locate 6,1;
     "Value Used:" hessian[seli,seli];
   seli = seli+1;
endo;

/* Now compute off-diagonal elements
** Make sure that correlations are between -1 and 1
** errorij contains the index of elements that are invalid
*/
errorij = 0~0~0;

seli = 1;
do until seli == npara;
   selj = seli+1;
   do until selj > npara;
     locate 1,1;
     "Hessian Element    (" seli selj ")";
     i=1;
     do until i > ndx;
      paradx = para;
      parady = para;
      paradx[seli] = paradx[seli] + dx[i]*dxscale[seli];
      parady[selj] = parady[selj] - dx[i]*dxscale[selj];
      paradxdy = paradx;
      paradxdy[selj] = paradxdy[selj] - dx[i]*dxscale[selj];
      fx  = fcn(para);
      fdx = fcn(paradx);
      fdy = fcn(parady);
      fdxdy = fcn(paradxdy);
      gradx[i] = -( fx - fdx )/ (dx[i]*dxscale[seli]);
      grady[i] = ( fx - fdy )/ (dx[i]*dxscale[selj]);
      gradxy[i] = -(fx -fdxdy)/ sqrt( (dx[i]*dxscale[selj])^2 + (dx[i]*dxscale[seli])^2 );
      hessdiag[i] = -( 2*fx - fdx - fdy)/(dx[i]*dxscale[seli])^2; 
      hessdiag[i] = -( fx - fdx - fdy + fdxdy )/(dx[i]*dx[i]*dxscale[seli]*dxscale[selj]);
      i = i+1;
     endo;
     "Values";
     -hessdiag';

     hessian[seli,selj] = -0.5*(hessdiag[3]+hessdiag[4]);
     
     if ( hessian[seli,seli] == 0 ) or (hessian[selj,selj] == 0);
        corrij = 0;
     else;
        corrij = hessian[seli,selj]/sqrt(hessian[seli,seli]*hessian[selj,selj]);
     endif;

     if (corrij < -1) OR (corrij > 1);
        hessian[seli,selj]=0;
        errorij = errorij |(seli~selj~corrij);
     endif;   
     hessian[selj,seli] = hessian[seli,selj];

     locate 6,1;
     "Value Used: " hessian[seli,selj];
     "Correlation:" corrij;
     "Number of Errors:" rows(errorij)-1;
     selj=selj+1;
   endo;
   seli = seli+1;
endo;

cls;
"Errors"  errorij;
"Hessian";

hessian[1:npara,1:npara];

/* Save Hessian to disk
*/
save path = ^lpath hessian;

/*******************************************************************************
*/

/* Load Hessian, compute penalty
*/
evalhess:
clear hessian;

loadm path = ^lpath hessian;
HHm = hessian;

invHHM  = inv(HHM);
sigmult = cholpsd(invHHM);
invHHMdet = det(invHHM);

"Determinant of minus Hessian";
invHHMdet;
"sqrt(Diagonal of Inverse Hessian)";
sqrt(diag(invHHM));

"Post Mode Penalty";
penalt = (rank(HHM)/2)*ln(2*pi) + 0.5*ln(invHHMdet);
penalt;

/* Save Multiplication Output files
*/
save path = ^lpath sigmult;
end;


/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (1) = fcn(para);
/* Evaluate the product of likelihood and prior
*/
local gam1_m, gam11_v, gam0_m, gam00_v, gam01_v, lnpy, gam1_mT, gam11_vT, lnprio;

  {gam1_m, gam11_v, gam0_m, gam00_v, gam01_v} = qdiff_gam(para, gamss_m0, gamss_v0, mspec); 
  {lnpy, gam1_mT, gam11_vT} = loglhpanel( para, beta_m0, beta_v0, gam1_m, gam11_v);
  
  /* Evaluate the Prior density
  */      
  lnprio = priodens( para, pmean, pstdd, pshape);
    
retp(real(lnpy+lnprio));  /* We minize the inverse of the likelihood fcn */
endp;


