/* filename:    loaddata.g
** description: loads the PSID data subset into Gauss
**              and computes sufficient statistics
*/

new;
closeall;
cls;

/********************************************************
**      Select number of individuals:
**      first n, if n=0 then use all
*/
nindividuals = 0;

/********************************************************
**      Select specification:
**      mspec = 1:  no covariates
**      mspec = 2:  covariates: AGE, AGE^2, EDUC, FEMALE
*/
#include c:\projects\active\persist\gauss\prog_p03\confspec.g;


/********************************************************
**      Read Panel Data from Disk
**
*/
lpath = "c:\\projects\\active\\persist\\data_p\\";
nread    = 1000;

/* Read Panel-Id 
*/
ldata       = lpath $+ "lbdid";
{id, id_names} = readdata(ldata,nread);

/* Read Panel Individual Characteristics
*/
ldata       = lpath $+ "lbdchar";
{xchar, xchar_names} = readdata(ldata,nread);
"Data Series Read";
$xchar_names';
"";

/* Read Panel Wages
*/
ldata       = lpath $+ "lbdwages";
{wage, wage_names} = readdata(ldata,nread);
"Data Series Read";
$wage_names';
"";
  
/* Read Panel Hours
*/
ldata       = lpath $+ "lbdhours";
{hours, hours_names} = readdata(ldata,nread);
/* Eliminate contemporaneous hours
*/
hours = hours[.,2:cols(hours)];
hours_names = hours_names[2:rows(hours_names)];
"Data Series Read";
$hours_names';
"";

/* Read Panel Exogenous Variables
*/
ldata       = lpath $+ "lbdydum";
{ydummies, ydummies_names} = readdata(ldata,nread);
"Data Series Read";
$ydummies_names';
"";

/* Read Panel Ti
*/
ldata       = lpath $+ "lbdti";
{ti, ti_names} = readdata(ldata,nread);
"Data Series Read";
$ti_names';
"";

"Press Key to Continue";
k = keyw;
cls;

/********************************************************
**      Compute "sufficient statistics" of the data
**
*/
"Computing Sufficient Statistics";

nchar = cols(xchar);
nydummies = cols(ydummies);

w = wage~hours;
w_names = wage_names | hours_names;

if mspec == 1;
   x = ydummies;
   x_names = ydummies_names;
else;
   x = xchar~ydummies;
   x_names = xchar_names | ydummies_names;
endif;

clear wage, hours, ydummies, xchar;

ww = w'*w;
xx = x'*x;
xw = x'*w;

nx = cols(x);
nw = cols(w);

if nindividuals == 0;
   nindividuals = rows(Ti);
endif;

wiiw = zeros(nindividuals,nw^2);
xiiw = zeros(nindividuals,nx*nw);
xiix = zeros(nindividuals,nx^2);
iw   = zeros(nindividuals,nw);
ix   = zeros(nindividuals,nx);

individual_pointer = 1;
i = 1;
do until i > nindividuals; 
   locate 3,1;
   "Individual" i;
   ind_T1 = individual_pointer;
   ind_Ti = individual_pointer+Ti[i]-1;
   wiiw[i,.] = vec( w[ind_T1:ind_Ti,.]'*ones(Ti[i],Ti[i])*w[ind_T1:ind_Ti,.])'; 
   xiiw[i,.] = vec( x[ind_T1:ind_Ti,.]'*ones(Ti[i],Ti[i])*w[ind_T1:ind_Ti,.])'; 
   xiix[i,.] = vec( x[ind_T1:ind_Ti,.]'*ones(Ti[i],Ti[i])*x[ind_T1:ind_Ti,.])'; 
   iw[i,.]   = sumc( w[ind_T1:ind_Ti,.] )';
   ix[i,.]   = sumc( x[ind_T1:ind_Ti,.] )';
   individual_pointer = individual_pointer + Ti[i];
   i = i+1;
endo;

ti   = ti[1:rows(wiiw)];

spath = lpath $+ "spec" $+ ftos(mspec,"%*.*lf",1,0);
save path = ^spath wiiw, xiiw, xiix, iw, ix,
                   ww, xx, xw, w_names, x_names, ti,
                   nchar, nydummies;

end;

/********************************************************************
*/

proc(2) = readdata(ldata,nread);
local fhdata, eofloop, dataset,
      dataset_names, datarows;

open fhdata = ^ldata for read;

eofloop  = 0;
dataset       = readr(fhdata, 1);
dataset_names = getname(ldata);

do until eofloop; 

   datarows = readr( fhdata, nread );
   dataset = dataset | datarows;
   eofloop = eof(fhdata);

endo;
closeall fhdata;

retp(dataset, dataset_names);
endp;


