/* filename:    postgam.g
** description: The program computes posterior means and variances conditional
**              on the posterior draws of RHO MU PHI SIG
**              
** created:     06/01/01
** modified:   
*/

new;
closeall;
cls;
outwidth 128;
library user, pgraph, lbdlib;
__output = 0;

npara = 4;
mhrun = "1";

/********************************************************
**      List of Global Variables
**
**      ww, xx, xw, wiiw, xiix, xiiw, iw, ix, w_names, x_names, ti
**      nx, nw, nchar, nydummies
**      gamss_m0, gamss_v0, beta_m0, beta_v0
**      age_base, educ_base, gender_base, hours_base, xbaseline
*/
#include c:\projects\active\persist\gauss\prog_p03\confspec.g;

/********************************************************
**      Load "sufficient statistics" for panel data set
**
*/
lpath = "c:\\projects\\active\\persist\\data_p\\spec" $+ ftos(mspec,"%*.*lf",1,0);
loadm path = ^lpath wiiw, xiiw, xiix, iw, ix,
                    ww, xx, xw, w_names, x_names, ti,
                    nchar, nydummies;
nx = cols(xx);
nw = cols(ww);
"Data Matrices successfully loaded";
"";

/********************************************************
**      Configure the prior for the nuisance parameters
**
**      specify baseline ind: age_base, educ_base, gender_base, hours_base     
**                            xbaseline
**      individual effect:    beta_m0, beta_v0
**      covariates       :    gamss_m0, gamss_v0
*/
#include c:\projects\active\persist\gauss\prog_p03\confprio.g;
gamss_v0 = gamss_v0*scale_g^2;
beta_v0  = beta_v0*scale_b^2;
"Prior successfully configured";
"";

/********************************************************
** Load Prior for Structural Parameters
*/
lpath = "c:\\projects\\active\\persist\\gauss\\prog_p03\\para";
priorfile = "panelpri.out";
load path=^lpath prior[npara,3] = ^priorfile;

pmean  = prior[.,1];
pstdd  = prior[.,2];
pshape = prior[.,3];


/********************************************************
**  Print Posterior Mode Information
*/
loadm path=^lpath para_names, lbdmod_p;

"Parameter   | Estimate ";
outmat = para_names~lbdmod_p;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);

"";
"Press Key to Continue";
k = keyw;
cls;


/********************************************************
**  Open the file with the parameter draws and 
**  initialize the output files
*/
lpath = "c:\\projects\\active\\persist\\gauss\\prog_p03\\results\\";
lpara  = lpath $+ "mp_par" $+mhrun;
open fhpara = ^lpara for read;

pararow = readr(fhpara,1);
nburn   = 10001;
pararow = seekr(fhpara,nburn);

/* Initialize Output files
*/
ngam = nx + 1;
opath = "c:\\projects\\active\\persist\\gauss\\prog_p03\\results\\";
ogam_m  = opath  $+ "gam_m" $+ mhrun;
ogam_v  = opath  $+ "gam_v" $+ mhrun;

create fhgam_m=^ogam_m with GAM_M, ngam+2, 8;
create fhgam_v=^ogam_v with GAM_V, ngam^2+4, 8;

eofloop = 0;
nread   = 5000;
rowsel  = 100;
loopct  = 0;

do until eofloop; 

   parasim = readr( fhpara, nread);
   blocksize = rows(parasim);
   parasel = ( (seqa(1,1,blocksize) % rowsel) .NE 0);

   if minc(parasel) == 0;

   parasim = delif(parasim,parasel);
   nused  = rows(parasim);
   gamsim_m = zeros(nused,ngam+2);
   gamsim_v = zeros(nused,ngam^2+4);

   i = 1;
   do until i > nused;

      paraest = parasim[i,.]';

      {gam1_m0, gam11_v0, gam0_m0, gam00_v0, gam01_v0} 
        = qdiff_gam(paraest, gamss_m0, gamss_v0, mspec); 
      {lnpy, gam1_mT, gam11_vT}   
        = loglhpanel( paraest, beta_m0, beta_v0, gam1_m0, gam11_v0);
      {gamss_mT, gamss_vT, AEP_mT, AEP_vT} 
        = qdiffinv_gam(paraest, gam1_mT, gam11_vT, gam0_m0, gam00_v0, gam1_m0, gam11_v0, gam01_v0, mspec);

      gamsim_m[i,.] = AEP_mT' ~ gamss_mT';
      gamsim_v[i,.] = vec(AEP_vT + AEP_mT*AEP_mT')' ~ vec(gamss_vT + gamss_mT*gamss_mT')'; 
      
      i = i+1;
   endo;

   wr = writer(fhgam_m,gamsim_m);
   wr = writer(fhgam_v,gamsim_v);

   endif;

   loopct = loopct + 1;
   locate 1,1;
   "Iteration: " loopct;
   "Blocksize: " blocksize;
   "Draws Used:" nused;

   eofloop = eof(fhpara);

endo;

closeall fhpara, fhgam_m, fhgam_v;


