/* filename:    postgam2.g
** description: The program computes posterior means and variances conditional
**              on the posterior draws of RHO MU PHI SIG
**              
** created:     06/01/01
** modified:   
*/

new;
closeall;
cls;
outwidth 128;
library user, pgraph, lbdlib;
__output = 0;

npara = 4;
mhrun = "1";

/********************************************************
**      List of Global Variables
**
**      ww, xx, xw, wiiw, xiix, xiiw, iw, ix, w_names, x_names, ti
**      nx, nw, nchar, nydummies
**      gamss_m0, gamss_v0, beta_m0, beta_v0
**      age_base, educ_base, gender_base, hours_base, xbaseline
*/
#include c:\projects\active\persist\gauss\prog_p03\confspec.g;

/********************************************************
**      Load "sufficient statistics" for panel data set
**
*/
lpath = "c:\\projects\\active\\persist\\data_p\\spec" $+ ftos(mspec,"%*.*lf",1,0);
loadm path = ^lpath wiiw, xiiw, xiix, iw, ix,
                    ww, xx, xw, w_names, x_names, ti,
                    nchar, nydummies;
nx = cols(xx);
nw = cols(ww);
"Data Matrices successfully loaded";
"";

/********************************************************
**      Configure the prior for the nuisance parameters
**
**      specify baseline ind: age_base, educ_base, gender_base, hours_base     
**                            xbaseline
**      individual effect:    beta_m0, beta_v0
**      covariates       :    gamss_m0, gamss_v0
*/
#include c:\projects\active\persist\gauss\prog_p03\confprio.g;
gamss_v0 = gamss_v0*scale_g^2;
beta_v0  = beta_v0*scale_b^2;
"Prior successfully configured";
"";
"Press Key to Continue";
k = keyw;
cls;

/********************************************************
**  Open the file with the parameter draws and 
**  initialize the output files
*/
lpath = "c:\\projects\\active\\persist\\gauss\\prog_p03\\results\\";
lgam_m  = lpath $+ "gam_m" $+mhrun;
lgam_v  = lpath $+ "gam_v" $+mhrun;
open fhgam_m = ^lgam_m for read;
open fhgam_v = ^lgam_v for read;

gamsim_mc = readr(fhgam_m,1);
gamsim_vc = readr(fhgam_v,1);
ngam = nx+1;

eofloop = 0;
nread   = 5;
ndraws  = 1;
loopct  = 0;

do until eofloop; 

   gamsim_m = readr(fhgam_m,nread);
   gamsim_v = readr(fhgam_v,nread);

   gamsim_mc = gamsim_mc + sumc(gamsim_m)';
   gamsim_vc = gamsim_vc + sumc(gamsim_v)';
   
   ndraws = ndraws + rows(gamsim_m);

   loopct = loopct + 1;
   locate 1,1;
   "Iteration: " loopct;

   eofloop = eof(fhgam_m);

endo;

closeall fhgam_m, fhgam_v;
"Gamma Draws have been loaded - Press Key to Continue";
k = keyw;
cls;

gamsim_m = gamsim_mc/ndraws;
gamsim_v = gamsim_vc/ndraws;

AEP_mT      = gamsim_m[1:2]';
AEPscale_mT = gamsim_m[1];
AEPpeak_mT  = gamsim_m[2];

AEP_vT = reshape(gamsim_v[1:4],2,2) - AEP_mT*AEP_mT';
AEPscale_sT = sqrt(AEP_vT[1,1]);
AEPpeak_sT  = sqrt(AEP_vT[2,2]);

gamss_mT    = gamsim_m[3:ngam+2]';
gamss_vT    = reshape(gamsim_v[5:4+ngam^2],ngam,ngam) - gamss_mT*gamss_mT';

retedu_mT   = gamss_mT[3];
retedu_sT   = sqrt(gamss_vT[3,3]);

gender_mT   = gamss_mT[4];
gender_sT   = gamss_vT[4,4];

lwstar_mT   = gamss_mT[5:4+nYDUMMIES+1];
lwstar_m0   = gamss_m0[5:4+nYDUMMIES+1];
lwstar_sT   = sqrt(diag(gamss_vT[5:4+nYDUMMIES+1,5:4+nYDUMMIES+1]));
lwstar_s0   = sqrt(diag(gamss_v0[5:4+nYDUMMIES+1,5:4+nYDUMMIES+1]));

wstar_mT    = exp(lwstar_mT);
wstar_m0    = exp(lwstar_m0);
wstar_sT    = sqrt( exp(lwstar_mT) ) .* lwstar_sT;
wstar_s0    = sqrt( exp(lwstar_m0) ) .* lwstar_s0;

wstar_b0    = wstar_m0-2*wstar_s0 ~ wstar_m0+2*wstar_s0;
wstar_bT    = wstar_mT-2*wstar_sT ~ wstar_mT+2*wstar_sT;

spara_names = "AEPscale" |
              "AEPpeak " |
              "Ret(Edu)" |
              "FEMALE  " ;

"Parameter   | Post Mean |  Post Std  | Prior Mean  | Prior Std";
outmat = spara_names 
       ~ (AEPscale_mT | AEPpeak_mT | retedu_mT | gender_mT )
       ~ (AEPscale_sT | AEPpeak_sT | retedu_sT | gender_sT ) 
       ~ (AEPscale_m  | AEPpeak_m  | retedu_m  | gender_m  )
       ~ (AEPscale_s  | AEPpeak_s  | retedu_s  | gender_s  );

let mask[1,5] = 0 1 1 1 1;
let fmt[5,3] =
   "-*.*s " 10 10
   "*.*lf " 10 4
   "*.*lf " 10 4
   "*.*lf " 10 4
   "*.*lf " 10 4;

d = printfm(outmat,(0 ~ 1 ~ 1 ~ 1 ~ 1),fmt);

tt = seqa(1976,1,nYDUMMIES+1);

   graphset;
   begwind;
   margin(0,0,0.2,0.2);
   window(2,1,0);
   fonts("microb");
   _ptitlht = 0.3;
   _paxht = 0.3;
   _pnumht = 0.25;
   _pcolor =2 ;
   _pltype = {6 6 2 2 };
   _protate = 1;
   _plctrl = 0;
   _plwidth = 5.5;
   _psymsiz = 5;

   _plegstr = "Posterior (L)\000Posterior (U)\000Prior (L)\000Prior (U)";
   _plegctl = {1 4 1977 14};
   ylabel("Efficiency Wages");   
   xlabel("Year");
   xy(tt, wstar_bT~wstar_b0);
   nextwind;
 
   endwind;

outmat = tt~wstar_mT~wstar_sT~wstar_m0~wstar_s0;

let mask[1,5] = 1 1 1 1 1;
let fmt[5,3] =
   "*.*lf " 10 4
   "*.*lf " 10 4
   "*.*lf " 10 4
   "*.*lf " 10 4
   "*.*lf " 10 4;

"Year  |  Post Mean | Post S.E. | Prior Mean | Prior S.E.";
d = printfm(outmat,mask,fmt);

