/* filename:    acfcomp.g
** description: Compute Prediction Losses for Moments
**
*/

new;
library user, pgraph;
cls;
outwidth 128;
format /mb1 /rds 10,4;


mhrun = "1";
lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
outfile = lpath $+ "acfcomp.out";
output file=^outfile reset;
output file=^outfile off; 

nblock  = 5000;
nburn   = 10001;

/* Set up loop to evaluate correlation predictions
*/

modelid     = "m11"  $| "m21" $| "m22";
corrid      = "cory" $| "corh"; 
lagid       = 4;

modelid_n = rows(modelid);
corrid_n  = rows(corrid);
lagid_n   = rows(lagid);


m = 1;
do until m > modelid_n;

   c = 1;
   do until c > corrid_n;

      l = 1;
      do until l > lagid_n;

         /* Load DSGE Model Posterior Mean Predictions
         */

         osummary = lpath $+ modelid[m] $+ corrid[c] $+ mhrun $+"s";
         open fhsummary = ^osummary for read;
         m_corrmean = readr(fhsummary,1);
         closeall fhsummary;
         
         m_corrmean = m_corrmean[1:lagid[l]];         

         /* Load VAR Posterior Mean Predictions
         */
         osummary = lpath $+ "m01" $+ corrid[c] $+ mhrun $+"s";
         open fhsummary = ^osummary for read;
         m0corrmean = readr(fhsummary,1);
         drawdim = cols(m0corrmean);
         m0corrstdd = readr(fhsummary,1);
         m0corrcov  = readr(fhsummary,drawdim);
         m0corrband = readr(fhsummary,2);

         m0corrmean = m0corrmean[1:lagid[l]];
         m0corrcov  = m0corrcov[1:lagid[l],1:lagid[l]];
         m0corrband = m0corrband[.,1:lagid[l]];
 
         chi2mc   = zeros(1,1);
         chi2pred =  - (m_corrmean-m0corrmean)*inv(m0corrcov)*(m_corrmean-m0corrmean)';
         chi2df   =  drawdim;
         chi2approx = 1 - cdfchic(-chi2pred,chi2df);
         l2       = (m_corrmean-m0corrmean)*(m_corrmean-m0corrmean)';

         
         /* Compute the expected chi2loss
         */

         eofloop = 0;
         ndraws  = 0;
         ldraws  = lpath $+ "m01" $+ corrid[c] $+ mhrun;
         open fhm0draws = ^ldraws for read;
         drawrow = seekr(fhm0draws, nburn);        

         do until eofloop; 

            drawblock = readr( fhm0draws, nblock );
            drawblock = drawblock[.,1:lagid[l]];
            ndraws = ndraws + rows(drawblock);

            i = 1;
            do until i > rows(drawblock);

              chi2draw = -(drawblock[i,.] - m0corrmean)*inv(m0corrcov)*(drawblock[i,.] - m0corrmean)';
              chi2mc   = chi2mc + (chi2draw .> chi2pred);
              i = i+1;

            endo;
            
            locate 1,1;
            "Draws" ndraws;
            eofloop = eof(fhm0draws);

         endo;
         closeall fhm0draws;

         chi2mc = chi2mc/ndraws;
         chi2mcerror = sqrt( (chi2mc - chi2mc^2)/ndraws );
 
         output file=^outfile on;
         "Prediction-losses";
         "=================================================";
         " ";
         "Model:             " $modelid[m];
         "Correlation:       " $corrid[c];
         "Number of Lags:    " lagid[l];
         " ";
         "Lag   |  VAR(mean) | VAR(band) |  DSGE(mean)";

         i = 1;
         do until i > lagid[l];

            outvec = i ~ m0corrmean[i] ~ m0corrband[.,i]' ~ m_corrmean[i];
            outvec;
            i = i+1;

         endo;            
         
         "";  
         "C(Chi2):           " -chi2pred;
         "L2-loss:           " l2;
         "Chi2-loss:         " chi2mc;
         "Chi2-loss (Approx):" chi2approx;
         "Chi2-loss (MC Err):" (100*chi2mcerror./chi2mc);
         "";
         "-------------------------------------------------";
         "";
         "";
         output file=^outfile off;
         l = l+1;
      endo;

      c = c+1;
   endo;

   m = m+1;
endo;        


