/***************************************************************/
/*                                                             */
/*      Compute Posterior Moments for IRFs and CORRs           */
/*                                                             */
/*                    Frank Schorfheide                        */
/*                                                             */
/***************************************************************/

/* Filename:  irfacmom.g
** created:   06/05/01
*/

new;
closeall;
library pgraph;
cls;


mhrun   = "1";
nburn   = 10001;
nblock  = 10000;

lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";

modelid  = "m01"  | "m11"  | "m21"  | "m22";
statname = "c11"  | "c12"  | "c21"  | "c22" | "cory" | "corh";
hpdprob  = 0.75   | 0.75   | 0.75   | 0.75  | 0.95   | 0.95;

nmodels = rows(modelid);
nstats  = rows(statname);

/* Loop to compute moments for the IRFs and CORRs
*/

i = 1;
do until i > nmodels;

   s = 1;
   do until s > nstats;

     /* open posterior draws
     */

     ldraw  = lpath $+ modelid[i] $+ statname[s] $+ mhrun;
     odraws = lpath $+ modelid[i] $+ statname[s] $+ mhrun $+ "s";
  
     open fhdraw = ^ldraw for read;

     drawrow = readr( fhdraw, nburn); 
     drawdim = cols(drawrow);
     drawrow = seekr( fhdraw, nburn);

     /* Part 1: Compute the mean of x(i) and x^2(i)
     */
     drawmean   = zeros(1,drawdim);
     drawsqmean = zeros(1,drawdim);
     drawcross  = zeros(drawdim,drawdim);

     eofloop = 0;
     ndraws  = 0;

     do until eofloop; 

       drawblock = readr( fhdraw, nblock );

       drawmean  = drawmean    + sumc(drawblock)';
       drawsqmean = drawsqmean + sumc(drawblock^2)';
       drawcross  = drawcross  + drawblock'*drawblock;  

       ndraws = ndraws + rows(drawblock);

       locate 1,1;
       "Draw-ID" $modelid[i] $statname[s];
       "Part 1 ";
       "Draws  " ndraws;

       eofloop = eof(fhdraw);

     endo;

     drawmean   = drawmean/ndraws;
     drawsqmean = drawsqmean/ndraws;
     drawstdd   = sqrt(drawsqmean - (drawmean)^2);
     drawcov    = drawcross/ndraws - drawmean'*drawmean;

     /* Part 2: HPD Interval
     */
     cls;
     drawci      = zeros(2,drawdim);

     j = 1;
     do until j > drawdim;

        /* Read only the j'th column
        */
        drawrow   = seekr( fhdraw, nburn); 
        drawblock = readr( fhdraw, 1);

        drawcol = drawblock[1,j];
        eofloop = 0;
        ndraws  = 1;
   
        do until eofloop; 

           drawblock = readr( fhdraw, nblock );
           drawcol   = drawcol | drawblock[.,j];
           ndraws    = ndraws + rows(drawblock);
           eofloop   = eof(fhdraw);
           locate 1,1;
           "Draw-ID" $modelid[i] $statname[s];
           "Part 2 ";
           "ndraws " ndraws;
           "Column " j;
         
        endo;
   
        drawci[.,j] = hpdint(drawcol,hpdprob[s]);
        j = j+1;

     endo;
     clear drawcol;

     create fhdraws = ^odraws with DRAWSUM, drawdim, 8;
     writer(fhdraws, real(drawmean | drawstdd | drawcov | drawci) );
     closeall fhdraws; 

     s = s+1;
   endo;
   
   i = i+1;
endo;


/**************************************************************/
/*                         Procedures                         */
/**************************************************************/

proc(1) = hpdint(draws,percent);
local     ndraws, drawdim, hpdband, nwidth, i, drawcoli,
          bup, minwidth, newwidth, j;

drawdim   = cols(draws);
ndraws    = rows(draws);
hpdband   = zeros(2,drawdim);
nwidth    = int(percent*ndraws);

i = 1;
do until i > drawdim;
   drawcoli = draws[.,i];
   /* sort response for period i, element 1 is max
   */
   drawcoli = rev(sortc(drawcoli,1));
   bup   = 1;
   minwidth  = drawcoli[1] - drawcoli[nwidth];
   j = 2;
   do until j > (ndraws-nwidth+1);
      newwidth = drawcoli[j] - drawcoli[j+nwidth-1];
      if newwidth < minwidth;
         bup = j;
         minwidth = newwidth;
      endif;
      j = j+1;
   endo;
   hpdband[2,i] = drawcoli[bup];
   hpdband[1,i] = drawcoli[bup+nwidth-1];
   i = i+1;
endo;
retp(hpdband);
endp;

