/***************************************************************/
/*                                                             */
/*            Convert Impulse Response Functions               */
/*                                                             */
/*                       Frank Schorfheide                     */
/*                                                             */
/***************************************************************/

/* Filename:  irfconv.g
** created:   07/17/00
*/
new;
closeall;
library pgraph;
cls;

mhrun   = "1";
lpath   = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
modelid = "m01" | "m11" | "m21" | "m22";
nmodels = rows(modelid);

nread = 10000;

/* Loop to Convert the impulse responses
** 
*/

i = 1;
do until i > nmodels;
 
   /* open raw responses
   */

   lir11 = lpath $+ modelid[i] $+ "ir11" $+mhrun;
   lir12 = lpath $+ modelid[i] $+ "ir12" $+mhrun;
   lir21 = lpath $+ modelid[i] $+ "ir21" $+mhrun;
   lir22 = lpath $+ modelid[i] $+ "ir22" $+mhrun;

   open fhir11 = ^lir11 for read;
   open fhir12 = ^lir12 for read;
   open fhir21 = ^lir21 for read;
   open fhir22 = ^lir22 for read;

   irfrow = readr(fhir11,1);
   nirf   = cols(irfrow);
   irfrow = seekr(fhir11,1);

   /* initialize output files for converted responses
   */
   oic11 = lpath $+ modelid[i] $+ "c11" $+ mhrun;
   oic12 = lpath $+ modelid[i] $+ "c12" $+ mhrun;
   oic21 = lpath $+ modelid[i] $+ "c21" $+ mhrun;
   oic22 = lpath $+ modelid[i] $+ "c22" $+ mhrun;
   oicxx = lpath $+ modelid[i] $+ "cxx" $+ mhrun;

   create fhic11 = ^oic11 with IR11, nirf, 8;
   create fhic12 = ^oic12 with IR12, nirf, 8;
   create fhic21 = ^oic21 with IR21, nirf, 8;
   create fhic22 = ^oic22 with IR22, nirf, 8;
   create fhicxx = ^oicxx with MULT, 2, 8;


   eofloop = 0;
   ndraws  = 0;
   loopct  = 0;

   do until eofloop; 

     irf11sim = readr( fhir11, nread);
     irf12sim = readr( fhir12, nread);
     irf21sim = readr( fhir21, nread);
     irf22sim = readr( fhir22, nread);
  
     /*
     mult1 = irf11sim[.,nirf];
     mult2 = irf22sim[.,1]   ;
     */

     mult1 = 0.01*abs(irf11sim[.,nirf])./irf11sim[.,nirf]; 
           /* Long-run response to permanent shock is positive */
     mult2 = 0.01*abs(irf22sim[.,3])./irf22sim[.,3];
           /* short-run response to transitory shock is positive */

     irf11sim = irf11sim./mult1;
     irf21sim = irf21sim./mult1;
     irf12sim = irf12sim./mult2;
     irf22sim = irf22sim./mult2;
  
     wr = writer(fhic11,irf11sim);
     wr = writer(fhic12,irf12sim);
     wr = writer(fhic21,irf21sim);
     wr = writer(fhic22,irf22sim);
     wr = writer(fhicxx,mult1~mult2);
   
     loopct = loopct + 1;
     ndraws = ndraws + rows(mult1);
     locate 1,1;
     "Model-Id:   " $modelid[i];
     "Iteration:  " loopct;
     "Total Draws:" ndraws;

     eofloop = eof(fhir11);

   endo;

   closeall fhir11, fhir12, fhir21, fhir22, 
            fhic11, fhic12, fhic21, fhic22, fhicxx;

   i = i+1;
endo;
