/******************************************************************/
/*                                                                */
/*         Compare IRFs and Corr for alternative parameters       */
/*                                                                */
/*                        Frank Schorfheide                       */
/*                                                                */
/******************************************************************/

/* filename: irfexpmt.g
** created:  10/13/99
** 
*/

new;
library lbdlib, user, pgraph;
cls;

mspec = 2;
nirf  = 40;                 /* number of impulse responses */
ncov  = 4;

irf_ti    = seqa(1,1,nirf);
cor_ti    = seqa(1,1,ncov);


irf_label = "Output / Trans Shock" $|
            "Hours / Trans Shock";

irf_yticinfo = ( -0.2 ~ 1 ~ 0.2 ~ 2 )|
               ( -0.2 ~ 1 ~ 0.2 ~ 2 );

/*******************************************************************
** Load VAR Responses
*/
mhrun = "1";
lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
lsummary       = lpath $+ "m01c12" $+ mhrun $+ "s";
open fhsummary = ^lsummary for read;
m0ic12         = readr(fhsummary,1)';
closeall fhsummary;

lsummary       = lpath $+ "m01c22" $+ mhrun $+ "s";
open fhsummary = ^lsummary for read;
m0ic22         = readr(fhsummary,1)';
closeall fhsummary;

/*******************************************************************
** Load Posterior Mean Parameter estimates for LBD Model and use as
** starting values
** alp, bet, gam, del, nu, rho, mu, phi, chi, psi, siga, sigb
**  1    2    3    4    5   6    7   8    9    10   11    12
*/
mhrun = "1";
lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\para";
load path = ^lpath para_names;

lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
osummary       = lpath $+ "m21par" $+ mhrun $+ "s";
open fhsummary = ^osummary for read;
m2para         = readr(fhsummary,1)';
npara          = rows(m2para);
closeall fhsummary;


m2para = ones(1,7).*. m2para;

/* Change parameters in column 2-4
*/
m2para[6,2] = 0.85;
m2para[6,3] = 0.5;
m2para[6,4] = 0.0;

/* Change parameters in column 5-7
*/
m2para[6,6] = 0.85;
m2para[6,7] = 0.8;
m2para[7,7] = 0.2;
m2para[8,7] = 0.7;

/********************************************************************
** Print the Estimation Results
*/
cls;

"Parameter   | Spec 1  |  Spec 2  | Spec 3";
outmat = para_names'~m2para;
let mask[1,8] = 0 1 1 1 1 1 1 1;
let fmt[8,3] =
   "-*.*s " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4;

d = printfm(outmat,mask,fmt);


/********************************************************************
** Compute Impulse response functions and correlations
** 12 22 21 22
*/

irfy = zeros(nirf,7);
irfh = zeros(nirf,7);

i = 1;
do until i > 7;

   {ssirfda, valid} = dsgeirf( m2para[.,i],mspec,nirf, (100*m2para[11,i]|0) );
   {ssirfdb, valid} = dsgeirf( m2para[.,i],mspec,nirf, (0|-100*m2para[12,i]) );

   irfy[.,i]  = ssirfdb[.,1];
   irfh[.,i]  = ssirfdb[.,3];
   
   i = i+1;

endo;

/********************************************************************
** Plot impulse response functions and autocorrelation functions
*/
   graphset;
   begwind;
   margin(0,0,0.2,0.2);
   window(3,2,0);
   fonts("microb");
   _ptitlht = 0.4;
   _paxht = 0.4;
   _pnumht = 0.3;
   _pcolor =2 ;
   _pltype = {6 1 2 3};
 /*  _pltype = {6 1 1 3}; */
   _protate = 1;
   _plctrl = 0;
   _plwidth = 3.5;
   _psymsiz = 5;

 
      title("Experiment 1");
      ylabel(irf_label[1]);  
      ytics(irf_yticinfo[1,1], irf_yticinfo[1,2], irf_yticinfo[1,3], irf_yticinfo[1,4]);   
      _plegstr = "LBD Prior 1\000RHO = 0.85\000RHO = 0.5\000RHO = 0.0";
      _plegctl = {2 4 2.7 5.2};
      _pltype = {1 2 3 4};
      _pline = (1 ~ 6 ~ 0 ~ 0 ~ 40 ~ 0 ~ 4 ~ 1);
      xy(irf_ti,irfy[.,1:4]);
      nextwind;

      title("Experiment 2");
      ylabel("");  
      ytics(irf_yticinfo[1,1], irf_yticinfo[1,2], irf_yticinfo[1,3], irf_yticinfo[1,4]);   
      _plegstr = "LBD Prior 1\000RHO = 0.85\000RHO = 0.8, MU = 0.2, PHI = 0.7\000VAR (Post. Mean)";
      _plegctl = {2 4 1.7 5.2};
      _pltype = {1 3 6 2};
      _pline = (1 ~ 6 ~ 0 ~ 0 ~ 40 ~ 0 ~ 4 ~ 1);
      xy(irf_ti,irfy[.,5:7]~m0ic12);
      nextwind;

      title("");
      ylabel(irf_label[2]);  
      ytics(irf_yticinfo[2,1], irf_yticinfo[2,2], irf_yticinfo[2,3], irf_yticinfo[2,4]);   
      _plegstr = "LBD Prior 1\000RHO = 0.85\000RHO = 0.5\000RHO = 0.0";
      _pltype = {1 2 3 4};
      _plegctl = {2 4 2.7 5.2};
      _pline = (1 ~ 6 ~ 0 ~ 0 ~ 40 ~ 0 ~ 4 ~ 1);
      xy(irf_ti,irfh[.,1:4]);
      nextwind;

      title("");
      ylabel("");  
      ytics(irf_yticinfo[2,1], irf_yticinfo[2,2], irf_yticinfo[2,3], irf_yticinfo[2,4]);   
      _plegstr = "LBD Prior 1\000RHO = 0.85\000RHO = 0.8, MU = 0.2, PHI = 0.7\000VAR (Post. Mean)";
      _plegctl = {2 4 1.7 5.8};
      _pltype = {1 3 6 2};
      _pline = (1 ~ 6 ~ 0 ~ 0 ~ 40 ~ 0 ~ 4 ~ 1);
      xy(irf_ti,irfh[.,5:7]~m0ic22);
      endwind;

end;

/*************************************************************************/
/*                           PROCEDURES                                  */
/*************************************************************************/

proc (2) = dsgeirf( para,mspec,nsim,sig );
/* This procedure generates impulse response functions
*/
local valid, TTT, RRR, retcode, ss, irf, t, nvar, zt;

valid = 1;
/* Call procedure to solve the model
*/
{TTT, RRR, retcode} = lbdsolv(para,mspec);
if retcode > 0;
   valid = 0;
   irf = 0;
   goto eoirf;
endif;

nvar = cols(TTT);

ss = zeros(nsim+1,nvar);
zt = zeros(nsim+1,1);
irf = zeros(nsim+1,3);

ss[2,.] = ( TTT*ss[1,.]' + RRR*sig )';
zt[2,.] = zt[1,.] + (1~0)*sig;
irf[2,.] = (ss[2,1]+zt[2,.])~(ss[2,1]+zt[2,.]-ss[2,2])~ss[2,3];

t = 3;
do until t > nsim+1;
   ss[t,.] = ( TTT*ss[t-1,.]' )';
   zt[t,.] = zt[t-1,.];
   irf[t,.] = (ss[t,1]+zt[t,.]) ~ ( ss[t,1]-ss[t,2]) ~ ss[t,3];
   t = t+1;
endo;

irf = irf[2:nsim+1,.];
eoirf:
retp(irf, valid);
endp;

