/****************************************************/
/*                                                  */
/* Computing Impulse Response Functions and Moments */
/* and Spectra based on parameters of DSGE model    */
/*                                                  */
/*                                                  */
/****************************************************/

/* filename:    lbdpred.g
** description: The program converts the DSGE parameters
**              into impulse responses, etc
** created:     21/10/99
*/

new;
closeall;
library lbdlib, pgraph, user;
cls;

mspec  = 2;
mprior = 2;
mhrun  = "1";

/* Open files with Parameter Draws
*/
npara = 12;
lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
opath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results\\";
lmodel = "m" $+ ftos(mspec,"%*.*lf",1,0);
lprior = ftos(mprior,"%*.*lf",1,0);

lpara = lpath $+ lmodel $+ lprior $+ "par" $+ mhrun;
open fhpara = ^lpara for read;

/* Define some Constants
*/
nblock  = 1000;         /* block size */
rowsel  = 1;

ncovout = 8;           /* number of lags for the autocovariance function */
nirf    = 40;

/* Initialize Output files
*/
oir11 = opath $+ lmodel $+ lprior $+ "ir11" $+ mhrun;
oir12 = opath $+ lmodel $+ lprior $+ "ir12" $+ mhrun;
oir21 = opath $+ lmodel $+ lprior $+ "ir21" $+ mhrun;
oir22 = opath $+ lmodel $+ lprior $+ "ir22" $+ mhrun;

ocory = opath $+ lmodel $+ lprior $+ "cory" $+ mhrun;
ocorh = opath $+ lmodel $+ lprior $+ "corh" $+ mhrun;

create fhir11 = ^oir11 with IR11, nirf, 8;
create fhir12 = ^oir12 with IR12, nirf, 8;
create fhir21 = ^oir21 with IR21, nirf, 8;
create fhir22 = ^oir22 with IR22, nirf, 8;

create fhcory = ^ocory with CORY, ncovout, 8;
create fhcorh = ^ocorh with CORH, ncovout, 8; 
 
/* Initialization
*/ 

eofloop = 0;
loopct  = 1;
totaldrawsused = 0;

do until eofloop; 

   /* Read blocks of size nblock
   */
   parasim = readr( fhpara, nblock );
   nread = rows(parasim);

   parasel = ( (seqa(1,1,nread) % rowsel) .NE 0);

   locate 1,1;
   "Block:        " loopct;
   "Size:         " nread;

   if minc(parasel) == 0;

     parasim = delif(parasim,parasel);
     nused   = rows(parasim);
     totaldrawsused = totaldrawsused + nused;

     "Elements Used:" nused;
     "";
     "Total Draws:  " totaldrawsused;

     Asim11 = zeros(nused,nirf);
     Asim12 = zeros(nused,nirf);
     Asim21 = zeros(nused,nirf);
     Asim22 = zeros(nused,nirf);

     CORYsim   = zeros(nused,ncovout);
     CORHsim   = zeros(nused,ncovout);

     j = 1;
     do until j > nused;
      /* For each element of the block, compute the desired statistic
      */

      siga = parasim[j,npara-1];
      sigb = parasim[j,npara  ];

      /* compute impulse responses for output hours
      */
      {irf1,valid} = dsgeirf( parasim[j,1:(npara-2)]', mspec, nirf, (siga|0)  );
      {irf2,valid} = dsgeirf( parasim[j,1:(npara-2)]', mspec, nirf, (0|sigb)  );
      Asim11[j,.] = irf1[.,1]';
      Asim21[j,.] = irf1[.,3]';
      Asim12[j,.] = irf2[.,1]';
      Asim22[j,.] = irf2[.,3]';

      /* Compute autocovariances and spectrum
      */
      {TTT, RRR, valid} = lbdsolv(parasim[j,.]', mspec);
      nstate = rows(TTT);
      /* State vector is of the form dy(t), dy(t-1), dlm(t), ...
      */
      ZZ = zeros(2,nstate);
      ZZ[1,1] = 1;
      ZZ[1,2] = -1;
      ZZ[2,3] = 1;
   
      HH = ( siga^2 ~ 0 )|( 0 ~ 0     );
      QQ = ( siga^2 ~ 0 )|( 0 ~ sigb^2); 
      VV = ( siga^2 ~ 0 )|( 0 ~ 0     );

      /* Initialize Autocovariances
      */
      GAMM0 = zeros(4,ncovout+1);
      CORR0 = zeros(4,ncovout+1);

      GA0  = inv(eye(nstate^2) - (TTT.*.TTT))*vec(RRR*QQ*RRR');
      GA0  = reshape(GA0,nstate,nstate);

      Gl   = ZZ*GA0*ZZ' + ZZ*RRR*VV + (ZZ*RRR*VV)' + HH;
      Gstar= ( Gl[1,1] ~ sqrt(Gl[1,1]*Gl[2,2]) )|
          ( sqrt(Gl[1,1]*Gl[2,2]) ~ Gl[2,2] );

      GAMM0[.,1] = vecr(Gl);
      CORR0[.,1] = vecr(Gl./Gstar);
 

      TTl = TTT;
      l=1;
      do until l > ncovout;
       Gl = ZZ*(TTl*GA0)*ZZ' + ZZ*(TTl*RRR*VV);
       GAMM0[.,l+1] = vecr(Gl);           
       CORR0[.,l+1] = vecr(Gl./Gstar);
       TTl = TTl*TTT;
       l = l+1; 
      endo;

      CORYsim[j,.] = CORR0[1,2:ncovout+1];
      CORHsim[j,.] = CORR0[4,2:ncovout+1];
      j = j+1;
   endo;

   /* Write Output
   */
   wr = writer(fhir11,Asim11);
   wr = writer(fhir12,Asim12);
   wr = writer(fhir21,Asim21);
   wr = writer(fhir22,Asim22);
   wr = writer(fhcory,CORYsim);
   wr = writer(fhcorh,CORHsim);

  endif;

  loopct = loopct + 1;
  eofloop = eof(fhpara);
endo;


closeall fhir11, fhir12, fhir21, fhir22,
         fhcory, fhcorh, fhpara;

/*******************************************************************
*/

proc (2) = dsgeirf( para,mspec,nsim,sig );
/* This procedure generates impulse response functions
*/
local valid, TTT, RRR, retcode, ss, irf, t, nvar, zt;

valid = 1;

/* Call procedure to solve the model
*/
{TTT, RRR, retcode} = lbdsolv(para,mspec);
if retcode > 0;
   valid = 0;
   irf = 0;
   goto eoirf;
endif;

nvar = cols(TTT);

ss = zeros(nsim+1,nvar);
zt = zeros(nsim+1,1);
irf = zeros(nsim+1,3);

ss[2,.] = ( TTT*ss[1,.]' + RRR*sig )';
zt[2,.] = zt[1,.] + (1~0)*sig;
irf[2,.] = (ss[2,1]+zt[2,.])~(ss[2,1]+zt[2,.]-ss[2,2])~ss[2,3];

t = 3;
do until t > nsim+1;
   ss[t,.] = ( TTT*ss[t-1,.]' )';
   zt[t,.] = zt[t-1,.];
   irf[t,.] = (ss[t,1]+zt[t,.]) ~ ( ss[t,1]-ss[t,2]) ~ ss[t,3];
   t = t+1;
endo;

irf = irf[2:nsim+1,.];
eoirf:
retp(irf, valid);
endp;


