/*************************************************************************
**  
**  This program tests the lbdsolv procedure
**  Author: Frank Schorfheide
**
**************************************************************************/

/* filename: testlbd.g
** created:  5/1/01
** Modifications:
**
*/
library pgraph, lbdlib;
cls;

/* Define model parameters
*/
alp = 0.7267;
bet = 0.99;
gam = 0.0052;
del = 0.025;
nu  = 2;
rho = 0.926;
mu  = 0.33;
phi = 0.90;
chi = 2;

gst = exp(gam);

para = alp | bet | gam | del | nu |rho | mu | phi | chi ;
mspec = 2;

{steadst,valid} = lbdsst(para,mspec);
yst = steadst[1];
cst = steadst[2];
ist = steadst[3];
kst = steadst[4];

{TTT, RRR, retcode} = lbdsolv(para,mspec);

if retcode == 1;
   "Invalid Specification";
   end;
elseif retcode == 2;
   "TH0 of Canonical System is not Invertible";
   end;
elseif retcode == 3;
   "No explosive roots";
   end;
elseif retcode == 4;
   "Incorrect Number of Explosive Roots";
   end;
elseif retcode == 5;
   "Solution not invertible";
   end;
endif;

/*
goto part2;
*/

/* PART 1: COMPUTE MOMENT STATISTICS
*/
part1:

nsim = 500;
nit  = 500;
ncor = 5;

siga = 0.0114;
sigb = 0.0096;

nstat = 4+ncor; 
simstat = zeros(nit,nstat);

i = 1;
do until i > nit;
         

   /* Call procedure to solve the model
   */
   {ss, valid} = lbdsim( para,mspec,nsim,(siga|sigb) );

   /* Output growth
   */
   dy =  ss[2:nsim,1]-ss[1:nsim-1,1];
   dc =  ss[2:nsim,2]-ss[1:nsim-1,2];
   di =  ss[2:nsim,3]-ss[1:nsim-1,3];

   sdely = stdc(dy);
   sdelc = stdc(dc);
   sdeli = stdc(di);
   slm   = stdc(ss[.,4]);

   cory = zeros(ncor,1);
   mdy   = meanc(dy);
   j = 1;
   do until j > ncor;
      cory[j] = 1/(nsim-j)*(dy[j:nsim-1]-mdy)'*(dy[1:nsim-j]-mdy) / (1/(nsim-1)*(dy[1:nsim-1]-mdy)'*(dy[1:nsim-1]-mdy));
      j=j+1;
   endo;

   simstat[i,.] = sdely ~ (sdelc/sdely) ~ (sdeli/sdely) ~ (slm/sdely) ~ cory';

   locate 1,1;
   "Iteration" i;
   i = i+1;
endo;

"Results";
"sdely, (sdelc/sdely), (sdeli/sdely), (slm/sdely)"
meanc(simstat[.,1:4]);
" ";
"Autocorrelations"
meanc(simstat[.,5:4+ncor]);

"Parameters";
para;


/* PART II: COMPUTE IMPULSE RESPONSE FUNCTIONS
*/
part2:

nirf = 40;
ti = seqa(1,1,nirf);

/* Simulate IRFs
*/
{ssirfdp, valid} = lbdirf( para,mspec,nirf,(1|0) );
{ssirfdk, valid} = lbdirf( para,mspec,nirf,(0|1) );


/* Plot Impulse Response Functions
*/
graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.8;
_pnumht = 0.4;
ymin = -5;
ymax =  5;
ytics(ymin,ymax,(ymax-ymin)/10,1);

titlstr = "dy  dy1 dl  dk  dx  dd  dc  db  de  ";

begwind;
window(3,4,0);
i = 1 ;
do until i > cols(ssirfdp);
   title("Technology Shock");
   ylabel(strsect(titlstr,1+(i-1)*4,4));
   xy(ti,ssirfdp[.,i]);
   nextwind;
   I = I + 1;
endo;
endwind;

ymin = -1;
ymax =  1;
ytics(ymin,ymax,(ymax-ymin)/10,1);

begwind;
window(3,4,0);
i = 1 ;
do until i > cols(ssirfdk);
   title("Preference Shock");
   ylabel(strsect(titlstr,1+(i-1)*4,4));
   xy(ti,ssirfdk[.,i]);
   nextwind;
   I = I + 1;
endo;
endwind;

end;

/******************************************/
/*          P R O C E D U R E S           */
/******************************************/

proc (2) = lbdsim( para,mspec,nsim,sig );
/* Procedure simulates sample path of DSGE model
** Returns:
**     model variables
**     valid
*/
local nvar,
      valid, retcode, TTT, RRR, ss, t, eps, gam,
      zt, steadst, yst, cst, ist, ysim, csim, isim, lsim;

gam = para[3];
valid = 1;

/* Call procedure to solve the model
*/
{TTT, RRR, retcode} = lbdsolv(para,mspec);
if retcode > 0;
   valid = 0;
   ss = 0;
   goto eosim;
endif;


/* Model Variables
** [dy(t), dy(t-1), dl(t), dk(t+1), dc(t), db(t), dx(t+1), dd(t), de(t) ]
*/
nvar = rows(TTT);

/* Simulate sample path
*/
ss = zeros(nsim,nvar);
zt = zeros(nsim,1);

t = 2;
do until t > nsim;
   eps = rndn(2,1);
   ss[t,.] = ( TTT*ss[t-1,.]' + RRR*( sig.*eps ) )';
   zt[t,.] = gam + zt[t-1,.] + eps[1]*sig[1];
   t = t+1;
endo;

/* Create output vector
*/
{steadst,valid} = lbdsst(para,mspec);
yst = steadst[1];
cst = steadst[2];
ist = steadst[3];

ysim = ln(yst) + zt + ss[.,1];
lsim = ln(para[5])+ss[.,3];
csim = ln(cst) + zt + ss[.,5];
isim = ln(yst-cst) + zt + yst/ist*ss[.,1] - cst/ist*ss[.,5];

ss = ysim ~ csim ~ isim ~ lsim;

eosim:
retp(ss, valid);
endp;


proc (2) = lbdirf( para,mspec,nsim,sig );
/* This procedure generates impulse response functions
*/
local valid, TTT, RRR, retcode, ssirf, t, nvar;

valid = 1;
/* Call procedure to solve the model
*/
{TTT, RRR, retcode} = lbdsolv(para,mspec);
if retcode > 0;
   valid = 0;
   ssirf = 0;
   goto eoirf;
endif;

nvar = cols(TTT);

ssirf = zeros(nsim,nvar);
ssirf[2,.] = ( TTT*ssirf[1,.]' + RRR*sig )';
t = 3;
do until t > nsim;
   ssirf[t,.] = ( TTT*ssirf[t-1,.]' )';
   t = t+1;
endo;

eoirf:
retp(ssirf, valid);
endp;
