/***************************************************************/
/*                                                             */
/*     VAR analysis: computing the posterior probability       */
/*                                                             */
/*                       Frank Schorfheide                     */
/*                                                             */
/***************************************************************/

/* Filename:  minvardd.g
** created:   11/03/98
** modified:  08/23/99 adapted to LBD data
*/
new;
closeall;
library pgraph, user;
cls; 

/* Set global variables to control the Gibbs sampler
*/
nsim   = 22000;
n_burn =  2000;

/* Import data on output growth and inflation: data_q (nobs,2)
** observations from 1954:III to 1997:IV
*/
#include c:\projects\active\persist\gauss\prog_t03\loaddat2.g;

/* Method: we compute the variance of the Minnesota prior based
** on the first T0(adjusted) observations. Then we combine this prior
** together with the training sample up to T0 to calculate the first
** predictive density
*/

   /* We start at period T0 to predict observation T0+1
   */
   t = T0;
   lnprob = 0;
   lnprob0 = 0;
   lndens = zeros(rows(ti),1);
   lndens0 = zeros(rows(ti),1);
  
   do until t == nobs;

      XXt     = XX[1:t,.];
      YYt     = YY[1:t,.];

      {lndens[t+1,.], lndens0[t+1,.]}
         = procpred(XXt, YYt, XX[t+1,.], YY[t+1,.], C_bar, CC_bar, VCm_inv);

      lnprob  = lnprob  + lndens[t+1,.];      
      lnprob0 = lnprob0 + lndens0[t+1,.];
      
      locate 1,1;
      "Iteration:   " t;
      "Predict Obs: " ti[t+1];
      "Data Density:" lnprob;
      
      t=t+1;

   endo;

/******************************************************************/
/*                  P R O C E D U R E S                           */
/******************************************************************/


proc(2) = procpred(XXt, YYt, X1, Y1, C_bar, CC_bar, VC_inv);
local t, i,
      p_sim, post, expo, j,
      CC_hat, Sig_hat, C_hat, Sig_inv, XXtXXt,
      VC_inv, VC_til, VC_tili, C_til, CC_til, C_sim, CC_sim,      
      H, Z, Sigi_sim, 
      nu, dens, lndens, sig_dens, lndens0, sim_error,
      k, nvar;


nvar   = cols(YY);
k = cols(XX);
t = rows(YYt);
lndens = 0;
dens = 1;
sig_dens= 0;
nu = 0;
sim_error =0;

/* Compute OLS estimates
*/
CC_hat  = inv(XXt'*XXt)*XXt'*YYt;
Sig_hat = (YYt-XXt*CC_hat)'*(YYt-XXt*CC_hat)/t;
C_hat   = vec(CC_hat);
Sig_inv = inv(Sig_hat);
XXtXXt  = XXt'*XXt;


  /* Draw initial psi from prior distribution
  */
  p_sim = p1_grid[floor(n_p1*rndu(1,1))+1];

  /* Draw initial c from approximate marginal
  */
  VC_inv  = VCm_inv/p_sim;
  VC_tili = VC_inv + (Sig_inv .*.XXtXXt);
  VC_til  = inv(VC_tili);
  C_til   = VC_til*( VC_inv*C_bar + (Sig_inv.*.XXtXXt)*C_hat );
  CC_til  = reshape(C_til,nvar,k)';
  C_sim   = C_til + cholpsd(VC_til)*rndn(nvar*k,1);
  CC_sim  = reshape(C_sim,nvar,k)';

  /* Draw initial sigi_sim
  */
  H = t*Sig_hat + (CC_sim - CC_hat)'*XXtXXt*(CC_sim - CC_hat);
  Z = cholpsd(inv(H))*rndn(nvar,t);
  Sigi_sim = Z*Z';

  lndens0 = -nvar*0.5*ln(2*pi) - 0.5*ln(det(Sig_hat))
            -0.5*(Y1 - X1*CC_hat)*inv(Sig_hat)*(Y1 - X1*CC_hat)';

  /* Start at max likelihood
  */
  CC_sim = CC_hat;
  C_sim  = C_hat;
  Sigi_sim = Sig_inv;
        
  i = 1;
  do until i > nsim;

     /* draw psi
     */
     post = zeros(n_p1,1);
     expo = 0.5*(C_sim - C_bar)'*VCm_inv*(C_sim - C_bar);
     j = 1;
     do until j > n_p1;
        post[j] = - 0.5*nvar*k*ln(p1_grid[j]) - expo/p1_grid[j];
        j = j+1;
     endo;
     post = post - ones(n_p1,1)*maxc(post);
     post = exp(post);
     post = post/sumc(post);
     z = rndu(1,1);
     j = 1;
     do until j > n_p1;
        if z le post[j];
           p_sim = p1_grid[j];
           j = n_p1+1;
        else;
           j = j+1;
           post[j] = post[j]+post[j-1];
        endif;
     endo;

     /* Draw C_sim
     */
     VC_inv  = VCm_inv/p_sim;
     VC_tili = VC_inv + Sigi_sim.*.XXtXXt;
     VC_til  = inv(VC_tili);
     C_til   = VC_til*(VC_inv*C_bar + (Sigi_sim.*.XXtXXt)*C_hat);
     CC_til  = reshape(C_til,nvar,k)';
     C_sim   = C_til + cholpsd(VC_til)*rndn(nvar*k,1);
     CC_sim  = reshape(C_sim,nvar,k)';

     /* Draw sigi_sim
     */
     H = t*Sig_hat + (CC_sim - CC_hat)'*XXtXXt*(CC_sim - CC_hat);
     Z = cholpsd(inv(H))*rndn(nvar,t);
     Sigi_sim = Z*Z';

     /* Compute expectation of log predictive density,
     ** forecast error and forecast error variance
     */
     if i == n_burn+1;
        dens = exp( -nvar*0.5*ln(2*pi) + 0.5*ln(det(Sigi_sim))
                 -0.5*(Y1 - X1*CC_sim)*Sigi_sim*(Y1 - X1*CC_sim)' );
        sig_dens = 0;
     elseif i > n_burn+1;
        nu     = exp (-nvar*0.5*ln(2*pi) + 0.5*ln(det(Sigi_sim))
                 -0.5*(Y1 - X1*CC_sim)*Sigi_sim*(Y1 - X1*CC_sim)' )
                 -dens;
        dens = dens + nu/(i-n_burn);
        sig_dens = sig_dens + nu^2/(1 + 1/(i-1-n_burn));
        sim_error = sqrt(sig_dens/(i-1-n_burn))/sqrt(i-1-n_burn);
     endif;
 
     lndens = ln(dens);
     
           
     locate 10,1;
     "Gibbs Sampler";
     "Iteration    " i;
     "Tightness    " p_sim;
     "Density score" dens ;
     "Sim std-error" sim_error;
     "Log-Density  " lndens;
      
     i = i+1;
   endo;

retp(lndens,lndens0);
endp;


