/*********************************************************
**                                                             
** Plot Posterior Density for MU and PHI                   
**
** Filename:  muphplt.g
** created:   05/31/01
*/
new;
library pgraph;
cls;
closeall;

/******************************************************** 
**      Specify the DSGE Model
**      Models: 1 = RBC
**              2 = LBD
**              3 = LBD + Effort
*/ 
lmodel = "m2";
npara  = 12;
mhrun  = "1";
pscale = 25;

nburn   = 10001;
nread   = 5000;
nsim    = 100000;
tailpr  = 0.10 | 0.9 ;

ngrid     = 151;
nocalc    = 0;
approxhpd = 0;

lossmu    = 0.3513;
lossphi   = 0.4080;

/******************************************************** 
** Load Prior for mu and phi
*/
ppath = "c:\\projects\\active\\persist\\gauss\\prog_p03\\results\\";

psummary = ppath $+ "mu_phiq1" $+"s";
open fhpsummary = ^psummary for read;
muphi_m0 = readr(fhpsummary,1)';
muphi_s0 = readr(fhpsummary,1)';
muphi_v0 = readr(fhpsummary,2);
closeall fhpsummar;

/******************************************************** 
** Load Parameter Draws, Prior 1
*/
lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results";
lpara = lpath $+ "\\" $+ lmodel $+ "1" $+ "par" $+ mhrun;

open fhpara = ^lpara for read;
drawrow     = seekr(fhpara,nburn);
parasim     = readr(fhpara,1);
muphi       = parasim[.,7:8];

eofloop = 0;
loopct  = 0;

do until eofloop; 

   pararow = readr( fhpara, nread);
   muphi   = muphi | pararow[.,7:8] ;

   loopct = loopct + 1;
   locate 1,1;
   "Loading Draws for LBD 1";
   "Iteration" loopct;

   eofloop = eof(fhpara);

endo;

closeall fhpara;

ndraws = rows(muphi);
"Number of Draws:" ndraws;
""
"Press key to continue";
k = keyw;
cls;

/******************************************************
** Calculate Mean and Variance
*/
muphi_mT1 = meanc(muphi);
muphi_vT1 = muphi'*muphi/ndraws - muphi_mT1*muphi_mT1';

/******************************************************** 
** Load Parameter Draws, Prior 2
*/
lpath = "c:\\projects\\active\\persist\\gauss\\prog_t03\\results";
lpara = lpath $+ "\\" $+ lmodel $+ "2" $+ "par" $+ mhrun;

open fhpara = ^lpara for read;
drawrow     = seekr(fhpara,nburn);
parasim     = readr(fhpara,1);
muphi       = parasim[.,7:8];

eofloop = 0;
loopct  = 0;

do until eofloop; 

   pararow = readr( fhpara, nread);
   muphi   = muphi | pararow[.,7:8];

   loopct = loopct + 1;
   locate 1,1;
   "Loading Draws for LBD(2)";
   "Iteration" loopct;

   eofloop = eof(fhpara);

endo;

closeall fhpara;

ndraws = rows(muphi);
"Number of Draws:" ndraws;
""
"Press key to continue";
k = keyw;
cls;

/******************************************************
** Calculate Mean and Variance
*/
muphi_mT2 = meanc(muphi);
muphi_vT2 = muphi'*muphi/ndraws - muphi_mT2*muphi_mT2';

/******************************************************
** Screen Output
*/

"Posterior Mean:";
"Prior 1" muphi_mT1';
"Prior 2" muphi_mT2';
""
"Posterior S.E.:";
"Prior 1:" sqrt(diag(muphi_vT1))';
"Prior 2:" sqrt(diag(muphi_vT2))';
"";
"Press Key to Continue";
k = keyw;
cls;

/******************************************************
** Simulate Draws from Prior / Posterior 2 and evaluate
** density
*/

/* Prior
*/
draws = rndn(nsim,2);
muphi_sim = muphi_m0' + pscale*draws*chol(muphi_v0);
muphidens_sim = sumc( (draws^2)' );
muphi_del = (muphi_sim[.,1] .< 0) .OR (muphi_sim[.,2] .< 0) .OR (muphi_sim[.,2] .GE 1);
muphidens_sim0 = delif(muphidens_sim,muphi_del);

/* Posterior
*/
muphi_sim = muphi_mT2' + draws*chol(muphi_vT2);
muphi_del = (muphi_sim[.,1] .< 0) .OR (muphi_sim[.,2] .< 0) .OR (muphi_sim[.,2] .GE 1);
muphidens_simT = delif(muphidens_sim,muphi_del);

clear draws, muphi_sim, muphi_del;

/* Multivariate Kernel estimate of posterior density
**
*/
"Evaluate Log-Density on Grid";


/* Define Grid
*/
mu_min1  = muphi_m0[1] - 3*sqrt(muphi_v0[1,1]);
mu_max1  = muphi_m0[1] + 6*sqrt(muphi_v0[1,1]);
phi_min1 = muphi_m0[2] - 6*sqrt(muphi_v0[2,2]);
phi_max1 = muphi_m0[2] + 3*sqrt(muphi_v0[2,2]);

mu_min2  = muphi_m0[1] - 1.5*pscale*sqrt(muphi_v0[1,1]);
mu_max2  = muphi_m0[1] + 3*pscale*sqrt(muphi_v0[1,1]);
phi_min2 = muphi_m0[2] - 3*pscale*sqrt(muphi_v0[2,2]);
phi_max2 = muphi_m0[2] + 1.5*pscale*sqrt(muphi_v0[2,2]);

mu_grd1   = seqa(mu_min1, (mu_max1-mu_min1)/(ngrid -1), ngrid);
phi_grd1  = seqa(phi_min1, (phi_max1-phi_min1)/(ngrid-1), ngrid);
mu_grd2   = seqa(mu_min2, (mu_max2-mu_min2)/(ngrid -1), ngrid);
phi_grd2  = seqa(phi_min2, (phi_max2-phi_min2)/(ngrid-1), ngrid);

phi_sel1  = (phi_grd1 .GE 0) .AND (phi_grd1 .LE 1);
phi_1ind1 = maxindc(phi_sel1)-1;
phi_1ind2 = maxindc(phi_sel1)+sumc(phi_sel1);

mu_sel1   = (mu_grd1 .GE 0);
mu_1ind   = maxindc(mu_sel1) - 1;

phi_sel2  = (phi_grd2 .GE 0) .AND (phi_grd2 .LE 1);
phi_2ind1 = maxindc(phi_sel2)-1;
phi_2ind2 = maxindc(phi_sel2)+sumc(phi_sel2);

mu_sel2   = (mu_grd2 .GE 0);
mu_2ind   = maxindc(mu_sel2) - 1;

if nocalc == 1;
   goto plothpd;
endif;

dens_01   = zeros(ngrid,ngrid);
dens_T1   = zeros(ngrid,ngrid);
dens_02   = zeros(ngrid,ngrid);
dens_T2   = zeros(ngrid,ngrid);


/* Compute density estimate
*/
y = 1;
do until y > ngrid;
   x = 1;
   do until x > ngrid;

      muphi_grd1 = mu_grd1[x] | phi_grd1[y];
      muphi_grd2 = mu_grd2[x] | phi_grd2[y];

      dens_01[y,x] = 100*(1-cdfchic( (muphi_grd1 - muphi_m0)'*inv(muphi_v0)*(muphi_grd1 - muphi_m0) , 2));
      dens_T1[y,x] = 100*(1-cdfchic( (muphi_grd1 - muphi_mT1)'*inv(muphi_vT1)*(muphi_grd1 - muphi_mT1) , 2));

      if approxhpd == 1;      
         dens_02val = 100*(1-cdfchic( (muphi_grd2 - muphi_m0)'*inv(muphi_v0*pscale^2)*(muphi_grd2 - muphi_m0) , 2));
         dens_T2val = 100*(1-cdfchic( (muphi_grd2 - muphi_mT2)'*inv(muphi_vT2)*(muphi_grd2 - muphi_mT2) , 2));
      else;
         dens_02val = 100 * meanc(muphidens_sim0 .< (muphi_grd2 - muphi_m0)'*inv(muphi_v0*pscale^2)*(muphi_grd2 - muphi_m0) );
         dens_T2val = 100 * meanc(muphidens_simT .< (muphi_grd2 - muphi_mT2)'*inv(muphi_vT2)*(muphi_grd2 - muphi_mT2) );
      endif;

      if (muphi_grd2[1] GE 0) AND (muphi_grd2[2] GE 0 ) AND (muphi_grd2[2] LE 1);
         dens_02[y,x] = dens_02val;
         dens_T2[y,x] = dens_T2val;
      else;
         dens_02[y,x] = 100;
         dens_T2[y,x] = 100;
      endif;

      x = x+1;
      locate 3,1;
      "Grid Point" x y;
   endo;
   y = y+1;
endo;

clear muphidens_sim0, muphidens_simT;

save path = ^lpath dens_01, dens_T1, dens_02, dens_T2;
                   
/* Create Contour Plot
*/
plothpd:
load path = ^lpath dens_01, dens_T1, dens_02, dens_T2;

graphset;
margin(0.1,0.1,0.1,0.1);
fonts("MICROB");
   _ptitlht = 0.3;
   _paxht = 0.25;
   _pnumht = 0.2;
   _pcolor = 0 ;
   _protate = 1;
   _pypmax = 3;
   _pxpmax = 3;
 /*  _plctrl = 4;*/
   _plwidth = 1.8;

 xlabel("MU(quarterly)");
 ylabel("PHI(quarterly)");
 _plev = 100*tailpr;

begwind;
window(2,1,0);

 _pzclr = 2;
 _pmsgctl = ( 0.115 ~ 0.82 ~ 0.3 ~ 0 ~ 1 ~ 2 ~ 1 )|
           ( 0.12  ~ 0.77 ~ 0.3 ~ 0 ~ 1 ~ 14 ~ 1 );

 xtics(mu_min1,mu_max1,sqrt(muphi_v0[1,1]),1);
 ytics(phi_min1,phi_max1,sqrt(muphi_v0[2,2]),1);

 _pmsgstr = "Prior 1\000Posterior 1";
 _pline   = ( 1 ~ 1 ~ mu_min1 ~ muphi_m0[2] ~ mu_max1 ~ muphi_m0[2]   ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ muphi_m0[1] ~ phi_min1 ~ muphi_m0[1] ~ phi_max1 ~ 4 ~ 1 );

 contour(mu_grd1',phi_grd1,dens_01);

setwind(1);

 _pzclr = 14;
 contour(mu_grd1',phi_grd1,dens_T1);

setwind(2);

 _pmsgctl = ( 0.05 ~ 0.9 ~ 0.3 ~ 0 ~ 1 ~ 2 ~ 1 )|
            ( 0.16 ~ 0.5 ~ 0.3 ~ 0 ~ 1 ~ 14 ~ 1 );

 xtics(mu_min2,mu_max2,5*sqrt(muphi_v0[1,1]),1);
 ytics(phi_min2,phi_max2,5*sqrt(muphi_v0[2,2]),1);

 ytics(0,1.1,0.1,0);
 _pzclr = 2;
 _pmsgstr = "Prior 2\000Posterior 2";
 _pline   = ( 1 ~ 1 ~ mu_min2 ~ muphi_m0[2] ~ mu_max2 ~ muphi_m0[2]   ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ muphi_m0[1] ~ phi_min2 ~ muphi_m0[1] ~ phi_max2 ~ 4 ~ 1 );
/* _psym    = (lossmu ~ lossphi ~ 10 ~ 4 ~ 4 ~ 1 ~ 5); */

 contour(mu_grd2',phi_grd2,dens_02);

setwind(2);

 _pzclr = 14;
 contour(mu_grd2',phi_grd2,dens_T2);

endwind;

